# -*- coding: utf-8 -*-
from paste.urlparser import StaticURLParser
from paste.deploy.converters import asbool, aslist
from paste.fileapp import FileApp
from webob import Request, Response, exc
from webob.dec import wsgify
from hgapp import config as hgconfig
from hgapp.ssh import keys
import subprocess
import logging
import shutil
import glob
import os

os.environ["HGENCODING"] = "utf-8"

import mercurial
from mercurial import hg
from mercurial import ui
from mercurial import config
from mercurial.localrepo import localrepository
from mercurial.hgweb import common
from mercurial.hgweb.hgwebdir_mod import hgwebdir
from mercurial.hgweb.hgweb_mod import hgweb
from mercurial.hgweb.hgweb_mod import perms
from mercurial.hgweb.request import wsgiapplication

try:
    from pitzw.app import ProjectApp
except ImportError:
    class ProjectApp(object):
        def __init__(self, *args, **kwargs):
            pass
        def __call__(self, environ, start_response):
            return HTTPNotFound()(environ, start_response)

from hgapp import hooks
from hgapp import utils
from hgapp.templates import render
from hgapp.templates import render_response

log = logging.getLogger(__name__)

class RepoApp(object):

    username = None
    is_dir = False

    def __init__(self, name, **defaults):
        self.pitzapps = {}
        section = hgconfig[name]
        name = name.split('hg:')[1]
        section.name = name
        section.prefix = name != 'main' and name or ''
        for k, v in defaults.items():
            if k not in section.keys():
                section[k] = v

        self.c = c = section

        if c.hgweb and c.hgwebdir:
            raise ValueError(
                    'You cant have a hgwebdir and hgweb in %s:%s' % (
                                    hgconfig.defaults()['__file__'], name))
        if not c.hgweb and not c.hgwebdir:
            raise ValueError(
                    'You must have a hgwebdir or hgweb in %s:%s' % (
                                    hgconfig.defaults()['__file__'], name))

        if c.hgweb:
            c.hgweb = os.path.realpath(c.hgweb)

        if c.hgwebdir:
            c.hgwebdir = os.path.realpath(c.hgwebdir)
            if not os.path.isdir(c.hgwebdir):
                os.makedirs(c.hgwebdir)

    def get_app(self, req):
        if req.path.startswith('/issues'):
            pitzapp = self.pitzapps.get(path, self.get_pitzapp(path))
            if pitzapp:
                req.path_info_pop()
                return pitzapp
            else:
                raise exc.HTTPNotFound()
        return self.get_repo_app(self.c.hgweb, req.hgapp_ui)

    def get_pitzapp(self, path):
        pitzdir = os.path.join(path, 'pitzdir')
        if os.path.isdir(pitzdir):
            perms = dict()
            for perm in ('view', 'comment', 'add'):
                perm = '%s_permission' % perm
                perms[perm] = str(self.c.get(perm, self.c.allow_read or '*'))
            for perm in ('edit',):
                perm = '%s_permission' % perm
                perms[perm] = str(self.c.get(perm, self.c.allow_push or 'authenticated'))
            pitzapp = ProjectApp(pitzdir, security=self.c.security or 'simple', **perms)
            if self.is_dir:
                self.pitzapps[path] = pitzapp
            else:
                self.pitzapps[None] = pitzapp
            return pitzapp

    def get_repo_app(self, path, baseui):
        c = self.c
        if utils.is_repository(path):
            repo = hg.repository(baseui, path)
            return hgweb(repo, os.path.basename(path))
        else:
            return exc.HTTPNotFound()

    def get_filename(self, req=None):
        path_info = req and req.path_info.strip('/') or 'index.html'
        if not path_info:
            path_info = 'index.html'
        repo_name = os.path.basename(self.c.hgweb)
        return utils.isfile(os.path.expanduser('~/.hgapp'),
                                'html', repo_name, *path_info.split('/'))

    def get_fileapp(self, req, filename):
        basename, ext = os.path.splitext(filename)
        if ext != '.html':
            # cache static stuff
            return FileApp(filename)
        elif req.remote_user:
            # cache anon content
            return FileApp(filename)
        else:
            # don't want cache. build our own response
            resp = Response()
            resp.content_type = 'text/html; charset=utf-8'
            fd = open(filename)
            resp.body = fd.read()
            fd.close()
            return resp

    def repo_edit(self, req, repo):
        if req.method == 'POST':
            kwargs = {}
            for k in ('contact', 'description', 'allow_read', 'allow_push'):
                if k in req.POST:
                    kwargs[k] = req.POST[k].strip()
            utils.edit_repo(repo.root, 'web', **kwargs)
            resp = exc.HTTPFound(location=req.application_url)
        else:
            package_name = os.path.basename(repo.root)
            forks=utils.find_forks(repo.root)
            resp = render_response('edit',
                                   package_name=package_name,
                                   forks=forks,
                                   repo=repo,
                                   req=req)
        return resp

    def __repr__(self):
        return '<%s %s>' % (self.__class__.__name__, self.c.hgweb)

    @wsgify
    def __call__(self, req):
        remote_user = req.remote_user
        if remote_user:
            req.remove_conditional_headers()

        cmd = req.GET.get('cmd', None)
        perm = perms.get(cmd, None)
        req.hgapp_app = self.__class__.__name__
        req.hgapp_config = self.c
        req.hgapp_ui = utils.get_ui(self.c)
        repo_req = req.copy()
        try:
            utils.check_perm(req, perm)
        except (exc.HTTPUnauthorized, exc.HTTPForbidden), resp:
            pass
        else:
            if remote_user:
                log.debug('access granted for %s', remote_user)
            filename = self.get_filename(repo_req)
            try:
                app = self.get_app(repo_req)
            except exc.WSGIHTTPException, resp:
                log.debug('no repo found at %s', req.path_info)
                pass
            else:
                repo = getattr(app, 'repo', None)
                if repo:
                    req.hgapp_repo = repo.path.split('/')[-2]
                    fork_path = repo.ui.config('paths', 'fork')
                    if fork_path:
                        req.hgapp_fork = fork_path
                        repo_req.hgapp_fork = fork_path

                    if remote_user:
                        # hgapp's repo actions
                        action = [p for p in repo_req.path_info.strip('/').split('/')][-1]
                        if action.startswith('repo.'):
                            meth = getattr(self, action.replace('.', '_'), None)
                            if meth is not None:
                                action_resp = meth(repo_req, repo)
                                if action_resp:
                                    return action_resp

                if not repo_req.content_type:
                    repo_req.content_type = ''

                try:
                    resp = repo_req.get_response(app)
                except Exception, e:
                    # give a chance to give a correct content_length to mercurial
                    repo_req.content_length = len(repo_req.body)
                    resp = repo_req.get_response(app)

                if not filename and self.is_dir and remote_user and \
                   resp.status_int == 404 and \
                   not repo_req.path_info.endswith('.html'):
                    if self.c.allow_web_init.as_bool('false'):
                        new_repo_resp = self.new_repo(req, resp)
                        if new_repo_resp:
                            return new_repo_resp

            if not cmd and filename:
                return self.get_fileapp(repo_req, filename)

        if resp.status[0:2] in ('40', '50'):
            resp.content_type = 'text/html'
            resp.body = render('error', status=resp.status,
                                        environ=repo_req.environ)

        return resp

class DirApp(RepoApp):

    is_dir = True

    def get_app(self, req):
        c = self.c
        baseui = req.hgapp_ui
        if os.path.isdir(c.hgwebdir):
            path_info = req.path_info.strip('/')
            if path_info.startswith(c.name):
                path_info = path_info.replace(c.name, '').strip('/')
            if path_info:
                path = os.path.join(c.hgwebdir, path_info.split('/')[0])
            else:
                path = None

            if path and utils.is_repository(path):
                # we are in a  repos
                req.path_info_pop()

                if req.path_info.startswith('/issues'):
                    pitzapp = self.pitzapps.get(path, self.get_pitzapp(path))
                    if pitzapp:
                        req.path_info_pop()
                        return pitzapp
                    else:
                        raise exc.HTTPNotFound()

                # return repos
                return self.get_repo_app(path, baseui)

            else:
                # load dir
                paths = glob.glob(os.path.join(c.hgwebdir, '*', '.hg'))
                paths = [os.path.split(p)[0] for p in paths]
                paths = sorted([(os.path.basename(p), p) for p in paths])
                return hgwebdir(paths, baseui=baseui)
        else:
            return exc.HTTPNotFound()

    def get_filename(self, req):
        path_info = req.path_info.strip('/')
        if '/' not in path_info:
            path_info += '/index.html'
        return utils.isfile(os.path.expanduser('~/.hgapp'), 'html',
                            *path_info.split('/'))

    def new_repo(self, req, resp):
        if len(req.path_info.split('/')) == 2:
            resp = render_response('new_repo',
                                   status=resp.status,
                                   **req.environ)
            return resp
        elif req.GET.get('hgapp.new_repo'):
            req.method = 'POST'
            try:
                utils.check_perm(req, 'push')
            except (exc.HTTPUnauthorized, exc.HTTPForbidden), resp:
                pass
            else:
                #create repo
                repo = os.path.join(self.c.hgwebdir,
                                    req.path_info.split('/')[1])
                utils.init_repo(repo, req.remote_user)
                resp = exc.HTTPFound(location=req.path_url)
                return resp

    def repo_fork(self, req, repo):
        if req.remote_user not in req.script_name:
            dest_dir = os.path.join(hgconfig['hg:config'].userdir,
                                    req.remote_user)
            if not os.path.isdir(dest_dir):
                os.makedirs(dest_dir)
            package_name = os.path.basename(repo.root)
            fork_name = req.GET.get('fork_name', package_name)
            dest_dir = os.path.join(dest_dir, fork_name)
            forks=utils.find_forks(repo.root)
            if forks:
                location='%s%s/%s/%s' % (
                                req.host_url, req.hgapp_root,
                                req.remote_user, fork_name)
                resp = render_response('fork',
                                       package_name=package_name,
                                       forks=forks,
                                       req=req)
            else:
                utils.fork_repo(
                        repo, dest_dir, req.remote_user,
                        req.script_name[len(req.hgapp_root):])
                resp = exc.HTTPFound(location='%s%s/%s/%s' % (
                                req.host_url, req.hgapp_root,
                                req.remote_user, fork_name))
            return resp

    def repo_delete(self, req, repo):
        if req.remote_user in req.script_name:
            package_name = os.path.basename(repo.root)
            if os.path.isdir(repo.root):
                shutil.rmtree(repo.root)
            resp = exc.HTTPFound(location='%s%s/%s/' % (
                            req.host_url, req.hgapp_root,
                            req.remote_user))
            return resp

    def __repr__(self):
        return '<%s %s>' % (self.__class__.__name__, self.c.hgwebdir)

class UserDirApp(DirApp):
    def __init__(self, name, *args, **kwargs):
        self.username = name
        name = 'hg:%s' % name
        super(UserDirApp, self).__init__(name, *args, **kwargs)

    @wsgify
    def __call__(self, req):
        if req.remote_user and \
           req.path_info in ('/prefs', '/prefs/') and \
           self.c.allow_web_keys.as_bool('false'):
            req.hgapp_repo = None
            if req.method == 'POST' and not req.GET.get('cmd'):
                if 'key_file' in req.POST:
                    fd = req.POST['key_file']
                    if hasattr(fd, 'file'):
                        keys.add_key(req.remote_user,
                                     name=req.POST['key_name'],
                                     key=fd.file)
                elif 'delete_keys' in req.POST:
                    user_keys = keys.user_keys(req.remote_user)
                    for name in req.POST.getall('delete_keys'):
                        if name in user_keys:
                            keys.del_key(req.remote_user, name)
                keys.authorized_keys()
                resp = exc.HTTPFound(location=req.path_url)
            else:
                user_keys = keys.user_keys(req.remote_user)
                resp = render_response('user_prefs', req=req,
                                       user_keys=user_keys)
            return resp
        return super(UserDirApp, self).__call__(req)

    def __repr__(self):
        return '<%s %s - %s>' % (self.__class__.__name__,
                                 self.username, self.c.hgwebdir)

