# -*- coding: utf-8 -*-
from hgapp import config
from hgapp import utils
import logging as log
import shutil
import sys
import os


def sphinx_update(ui, repo, hooktype=None, run_buildout=True, **kwargs):
    if hooktype != 'changegroup':
        return False

    if repo.ui.config('paths', 'fork'):
        log.warn('skipping sphinx hook (fork)')
        return False

    if not os.getenv('SSH_ORIGINAL_COMMAND'):
        if 'http' not in config['hg:config'].allow_docs and 'admin' not in kwargs:
            return False

    dirname = repo.root
    utils.cmd('hg', 'up', dirname=dirname)

    # try to get a sphinx config file
    for name in ('doc', 'docs'):
        conf = utils.isfile(dirname, name, 'conf.py')
        if conf:
            break

    if conf:
        # build docs

        docs = os.path.dirname(conf)

        package = os.path.basename(dirname)

        # set directorys
        builddir = config['hg:config'].get('builddir', os.path.expanduser('~/.hgapp'))
        htmldir = os.path.join(builddir, 'html', package)
        sphinxdir = os.path.join(builddir, package, 'sphinx')
        buildoutdir = os.path.join(builddir, package, 'buildout')

        # run buildout
        buildout = utils.isfile(dirname, 'buildout.cfg')
        if run_buildout and buildout:
            if not os.path.isdir(buildoutdir):
                os.makedirs(buildoutdir)
            utils.cmd('buildout', '-Nc', buildout,
                      'buildout:bin-directory=%s' % os.path.join(buildoutdir, 'bin'),
                      'buildout:eggs-directory=%s' % os.path.expanduser('~/eggs'),
                      'buildout:parts-directory=%s' % os.path.join(buildoutdir, 'parts'),
                      'buildout:develop-eggs-directory=%s' % os.path.join(buildoutdir, 'develop-eggs'),
                      dirname=dirname)

        # run sphinx
        args = ['-N', '-b', 'html']
        if 'admin' in kwargs:
            args.append('-E')

        doctrees = os.path.join(sphinxdir, 'build', 'doctrees')
        args.extend(['-d', doctrees, docs, htmldir])

        binary = utils.isfile(buildoutdir, 'bin', 'sphinx-build')
        kwargs = dict(dirname=dirname)
        if binary:
            kwargs['binary'] = binary
        else:
            args.insert(0, 'sphinx-build')
        utils.cmd(*args, **kwargs)
        log.info('sphinx docs updated at %s with %s', htmldir, binary or 'sphinx-build')
    return False

def sdist_update(ui, repo, hooktype=None, **kwargs):
    if hooktype != 'changegroup':
        return False
    dirname = repo.root
    utils.cmd('hg', 'up', dirname=dirname)
    if os.path.isfile(os.path.join(dirname, 'README.txt')):
        builddir = config['hg:config'].get('builddir', os.path.expanduser('~/.hgapp'))
        distdir = os.path.join(builddir, 'dist')
        kwargs = dict(dirname=dirname)
        args = ['setup', 'sdist', '-d', distdir]
        utils.cmd(*args, **kwargs)
        log.info('%s released', dirname)
    else:
        log.info('%s skipped. No README.txt found', dirname)

def updates(config, docs=False, sdists=False):
    from mercurial import hg
    for name in config.sections():
        if name.startswith('hg:'):
            section = config[name]
            dirname = None
            if name in ('config',):
                dirname = section.userdir
            else:
                dirname = section.hgwebdir
            if not dirname:
                continue
            if sdists:
                distdir = os.path.expanduser('~/.hgapp/dist')
                if os.path.isdir(distdir):
                    shutil.rmtree(distdir)
            for repo in os.listdir(dirname):
                path = os.path.join(dirname, repo)
                if utils.is_repository(path):
                    log.info("building", path)
                    ui = utils.get_ui(section)
                    repo = hg.repository(ui, path)
                    if docs:
                        sphinx_update(ui, repo, run_buildout=True, hooktype='changegroup',
                                      admin=True)
                    if sdists:
                        sdist_update(ui, repo, hooktype='changegroup',
                                      admin=True)
