# -*- coding: utf-8 -*-
from hgapp import utils
from hgapp import apps
from hgapp import config
from paste.urlparser import make_url_parser
from paste.fileapp import DirectoryApp
from ConfigObject import ConfigObject
from webob import Request, Response, exc
from webob.dec import wsgify
import logging
import os

log = logging.getLogger(__name__)

class Pool(dict):

    def __init__(self, global_conf, **local_conf):

        self.distdir = os.path.expanduser('~/.hgapp/dist')
        if not os.path.isdir(self.distdir):
            os.makedirs(self.distdir)
        self.distapp = DirectoryApp(self.distdir)

        self.config = config
        for name in config.sections():
            if name == 'hg:config':
                continue
            section = self.config[name]
            klass = section.hgwebdir and apps.DirApp or apps.RepoApp
            if name == 'hg:main':
                app = klass(name, allow_read='*')
                self[''] = app
            elif name.startswith('hg:'):
                app = klass(name, allow_read='*')
                prefix = '/'+app.c.prefix.strip('/')
                self[prefix] = app
                log.debug('%r at %s', app, prefix)
        self.userdir = config['hg:config'].userdir
        if self.userdir:
            if not os.path.isdir(self.userdir):
                os.makedirs(self.userdir)
            for username in os.listdir(self.userdir):
                self.add_user(username)

    def add_user(self, username):
        if username in ('config', 'main'):
            raise ValueError(username)
        name = 'hg:%s' % username
        if name not in self.config:
            section = self.config[name]
            home = os.path.join(self.userdir, username)
            section['hgwebdir'] = home
            section['sphinx_docs'] = os.path.join(self.userdir, '.sphinx', username)
            if not os.path.isdir(home):
                os.makedirs(home)
                cfg = ConfigObject()
                cfg.web = dict(allow_read='*', allow_push=username)
                fd = open(os.path.join(home, '.hgrc'), 'w')
                cfg.write(fd)
                fd.close()
            app = apps.UserDirApp(username, allow_read='*', allow_push=username)
            prefix = '/%s' % username
            self[prefix] = app
            log.debug('%r at %s', app, prefix)
            return app
        else:
            log.error('skipping %s user. a config section exist', username)

    def distindex(self):
        resp = Response()
        files = os.listdir(self.distdir)
        files = ''.join(['<div><a href="%s">%s</a></div>' % (f,f) for f in files])
        resp.body = '''<html>
        <head>
        <title>Available distributions</title>
        </head>
        <body>%s</body>
        </html>''' % files
        return resp

    @wsgify
    def __call__(self, req):
        req.hgapp_root = req.script_name
        path_info = req.path_info

        # auth stuff
        if path_info.startswith('/logout'):
            if req.remote_user:
                return exc.HTTPUnauthorized()
            else:
                path_info = req.path_info = '/'
        elif path_info.endswith('/login'):
            if not req.remote_user:
                return exc.HTTPUnauthorized()
            else:
                path_info = req.path_info = path_info.replace('/login', '')

        # dist stuff
        if path_info.startswith('/dist') and self.distapp:
            if path_info in ('/dist', '/dist/'):
                req.path_info = '/dist'
                return self.distindex()
            req.path_info_pop()
            resp = req.get_response(self.distapp)
            return resp

        # hgapp
        app = None
        for path in sorted(self.keys()):
            if path and (path == path_info or path_info.startswith(path+'/')):
                app = self[path]
                req.script_name += path
                path_info = path_info[len(path):]
                req.path_info = path_info
        if not app:
            user = req.remote_user
            if user and path_info.startswith('/%s' % user):
                app = self.add_user(user)
            else:
                app = self['']

        # various env var
        path_info = path_info.strip('/')
        req.hgapp_isdir = app.is_dir
        req.hgapp_path = str(app.c.hgwebdir) or str(app.c.hgweb)
        req.hgapp_username = app.username
        req.hgapp_repo = None
        req.hgapp_fork = None
        req.hgapp_ui = None

        return app

def make_app(global_conf, **local_conf):
    return Pool(global_conf, **local_conf)

