# -*- coding: utf-8 -*-
from optparse import OptionParser
from hgapp.pool import make_app
from hgapp import config
from hgapp import utils
from hgapp import hooks
from paste import httpserver
from sphinx import cmdline
from glob import glob
import pkg_resources
import logging as log
import sys
import os

def serve():
    log.setLevel(logging.DEBUG)
    ch = logging.StreamHandler()
    ch.setLevel(logging.DEBUG)
    formatter = logging.Formatter("%(levelname)s: %(message)s")
    ch.setFormatter(formatter)
    log.addHandler(ch)

    parser = OptionParser()
    parser.add_option("-w", "--www", dest="browser",
                      action="store_true", default=False,
                      help="open repos in a browser")
    parser.add_option("--host", dest="host",
                      action="store", default='127.0.0.1',
                      help="ip to bind to")
    parser.add_option("-p", "--port", dest="port",
                      action="store", default='5000',
                      help="port to bind")
    (options, args) = parser.parse_args()
    dirname = args and args[0] or os.getcwd()
    if os.path.isdir(os.path.join(dirname, '.hg')):
        kwargs = dict(hgweb=dirname)
    else:
        kwargs = dict(hgwebdir=dirname)
    kwargs.update(dict(private='false', allow='*'))
    app = make_hg({}, secure=False, **kwargs)
    if options.browser:
        import webbrowser
        webbrowser.open('http://%s:%s/' % (options.host, options.port))
    httpserver.serve(app, host=options.host,
                     port=int(options.port))

def main():
    if 'hg' in sys.argv:
        del sys.argv[1]
        pkg_resources.run_script('mercurial', 'hg')
    elif 'sphinx-build' in sys.argv:
        import sphinx
        del sys.argv[1]
        cmdline.main(sys.argv)
    elif 'buildout' in sys.argv:
        import zc.buildout.buildout
        del sys.argv[1]
        zc.buildout.buildout.main()
    elif 'setup' in sys.argv and os.path.isfile('setup.py'):
        del sys.argv[1]
        print os.getcwd()
        sys.path.insert(0, os.getcwd())
        __import__('setup')
    elif 'serve' in sys.argv:
        last_arg = sys.argv[-1]
        if not last_arg.endswith('hgapp.ini'):
            print 'Serving %s' % config.__file__
            sys.argv.append(config.__file__)
        import paste.script.command
        paste.script.command.run()
    else:
        serve()

def admin():
    parser = OptionParser()
    parser.add_option('-b', "--rebuild-docs", dest="docs",
                      action="store_true", default=False,
                      help="rebuild all docs")
    parser.add_option('-s', "--rebuild-sdists", dest="sdists",
                      action="store_true", default=False,
                      help="rebuild all sdists")
    (options, args) = parser.parse_args()

    if options.docs:
        print 'Updating docs...'
        hooks.updates(config, docs=True)

    if options.sdists:
        print 'Updating sdists...'
        hooks.updates(config, sdists=True)

def ssh_keys(args=None):
    parser = OptionParser()
    parser.add_option('-u', "--user", dest="user",
                      default=None,
                      help="user id")
    parser.add_option('-n', "--name", dest="name",
                      default=None,
                      help="name associate to ssh key")
    parser.add_option('-k', "--key", dest="key",
                      default=None,
                      help="path to an ssh pub key to add at NAME for USER")
    parser.add_option('-d', "--delete", dest="delete",
                      action="store_true", default=False,
                      help="delete key NAME for USER")
    parser.add_option('-l', "--list", dest="list",
                      action="store_true", default=False,
                      help="list ssh keys")
    parser.add_option('-r', "--regen", dest="regen",
                      action="store_true", default=False,
                      help="regen authorized_keys file")
    parser.add_option("-v", "--verbose", dest="verbose",
                      action="count", default=0)
    if args:
        (options, args) = parser.parse_args(args)
    else:
        (options, args) = parser.parse_args()

    log.basicConfig(stream=sys.stderr,
                    format='%(message)s',
                    level=options.verbose and log.DEBUG or log.INFO)

    from hgapp.ssh import keys

    def print_user(u):
        log.info(u)
        log.info('='*80)
        log.info('')
        for k, v in sorted(keys.user_keys(u).items()):
            log.info('%s' % k)
            log.info('  %s' % v)

    if options.list:
        if options.user:
            print_user(options.user)
        else:
            for u in keys.users():
                print_user(u)
        return

    if options.key:
        if options.user:
            keys.add_key(options.user, options.name, options.key)
            keys.authorized_keys()
            print_user(options.user)
            return
        else:
            parser.error('-u is required')

    if options.delete:
        if options.user and options.name:
            keys.del_key(options.user, options.name)
            keys.authorized_keys()
            print_user(options.user)
            return
        else:
            parser.error('-u and -n are required')

    if options.regen:
        keys.authorized_keys()
        return

    parser.parse_args(['-h'])
