# -*- coding: utf-8 -*-
from paste.urlparser import StaticURLParser
import saladedefruits as sdf
from saladedefruits.utils import Request
from hgapp.pool import make_app
from pyquery import PyQuery as pq
from webob import exc, dec
import mercurial
import traceback
import logging
import time
import sys
import os

log = logging.getLogger()

dirname = os.path.dirname(__file__)

class SkinApp(sdf.Saladier):

    mercurial_static = StaticURLParser(os.path.join(os.path.dirname(mercurial.__file__),
                                                    'templates', 'static'))
    @dec.wsgify(RequestClass=Request)
    def __call__(self, req):
        if req.method == 'POST' or req.GET.get('cmd'):
            try:
                resp = req.get_response(self.application)
            except Exception, e:
                tb = sys.exc_traceback
                log.critical('%s: %r\n%s\n%s', req.path_url, e,
                              req.environ, ''.join(traceback.format_tb(tb)))
                raise
            else:
                return resp
        elif '/static/' in req.path_info:
            req.path_info = req.path_info.split('/static')[1]
            req.script_name = ''
            return self.mercurial_static
        return sdf.Saladier.__call__(self, req)

class Salade(sdf.Salade):

    def __call__(self):
        req = self.request
        resp = self.response
        path_info = req.path_info.strip('/')

        head = self.content('head')
        head('link[type="text/css"]').remove()
        head('link[rel="icon"]').remove()

        self.standard_rules()

        app_title = self.theme('.mercurialapp-title')
        app_related = self.theme('.mercurialapp-related')
        app_content = self.theme('.mercurialapp-content')

        adhoc_attrs = resp.environ['webob.adhoc_attrs']
        repo = adhoc_attrs.get('hgapp_repo', '')
        template_title = self.content('h1.templateTitle')
        if template_title:
            package_name = template_title.html()
            self.theme('title').text(template_title.text())
            template_title.remove()
            root_url = None
        else:
            package_name = 'Repository at %s' % (req.script_name or '/',)
            root_url = None
            if repo and repo in req.path_info:
                package_name = repo
                if repo in path_info:
                    root_url = '%s/%s' % (req.application_url, repo)
                elif repo in req.script_name:
                    root_url = req.application_url

        username = req.remote_user
        if username:
            app_related.append('''
                 <li class="right"><a href="%(hgapp_root)s/logout">(logout)</a></li>
                 <li class="right"><a href="%(hgapp_root)s/%(username)s/prefs">%(username)s</a></li>
                 ''' % dict(adhoc_attrs, username=username))

        classes = dict(doc_class='', issues_class='', code_class='')
        if not self.content('.sphinxsidebar'):
            self.content('form').remove()
            for title in self.content('.title'):
                title = pq(title)
                if not title.text().split():
                    title.remove()
            self.content('div.page_header').remove()
            classes.update(code_class='active')
            app_content.addClass('body document')
            page_nav = self.content('.page_nav')
            if page_nav:
                app_content.before('<div class="related"><ul><li>%s</li></ul></div>' % page_nav.html())
                page_nav.remove()
            self.content('table').addClass('contentstable')
            self.content('.title').removeClass('title').addClass('mercurialapp-link').wrap('<h2></h2>')
            self.content('.parity0').removeClass('parity0').addClass('even')
            self.content('.parity1').removeClass('parity1').addClass('odd')
            self.content('.list').removeClass('list').addClass('mercurialapp-link')
            self.content('.page_footer').remove()
        else:
            self.content('div[style*="color"]').remove()
            sidebar = self.content('.sphinxsidebar')
            pq(self.content('.related')[0]).after(sidebar)
            if '/issues/' in path_info:
                classes.update(issues_class='active')
            else:
                classes.update(doc_class='active')

        format = dict(req.environ, package_name=package_name, **adhoc_attrs)
        if root_url:
            if username:
                clone_url = root_url.replace('://', '://%s@' % req.remote_user)
            else:
                clone_url = root_url
            format.update(root_url=root_url, clone_url=clone_url,
                          parent_url=req.application_url,
                          parent_path=req.script_name or '',
                          **classes)
            #    <li class="right"><a class="%(issues_class)s" href="%(root_url)s/issues/">issues</a></li>
            app_title.append('''<a  href="%(parent_url)s">%(parent_path)s/</a><a
                                    href="%(root_url)s/">%(package_name)s</a>''' % format)
            if req.remote_user:
                if req.remote_user not in req.script_name:
                    app_related.append('''
                        <li class="right">|</li>
                        <li class="right"><a class="fork" href="%(root_url)s/repo.fork">fork</a></li>
                        ''' % format)
                app_related.append('''
                    <li class="right">|</li>
                    <li class="right"><a class="delete" href="%(root_url)s/repo.edit">edit</a></li>
                    ''' % format)
            if adhoc_attrs.get('hgapp_fork'):
                fork = adhoc_attrs.get('hgapp_fork')
                app_related.append('''
                    <li class="right">|</li>
                    <li class="right"><a href="%(hgapp_root)s/%(hgapp_fork)s" title="Fork of %(hgapp_fork)s">main repo</a></li>
                    ''' % format)
            app_related.append('''
                <li class="right">|</li>
                <li class="right"><a class="%(code_class)s" href="%(root_url)s/summary">code</a></li>
                ''' % format)
            if not adhoc_attrs.get('hgapp_fork'):
                app_related.append('''
                    <li class="right">|</li>
                    <li class="right"><a class="%(doc_class)s" href="%(root_url)s/index.html">documentation</a></li>
                    ''' % format)
            app_related.append('''
                <li>hg clone <a href="%(clone_url)s/">%(clone_url)s/</a> </li>
                ''' % format)
        else:
            app_title.append('%(package_name)s' % format)


        app_content.append(self.content('body > *'))

        # set base url so we get full url in links
        if self.request.path_info.endswith('.html'):
            self.set_base_url('/'.join(self.request.path_url.split('/')[:-1]))
        else:
            self.set_base_url(self.request.path_url)

        self.rewrite_links()

        return self.theme


def make_skinned_app(global_conf, **local_conf):

    if 'theme' in local_conf:
        theme = local_conf.pop('theme')
    else:
        theme = os.path.join(dirname, 'static', 'theme.html')

    app = make_app(global_conf, **local_conf)

    return SkinApp(app, theme=theme, salade=Salade)

