# -*- coding: utf-8 -*-
from unittest import TestCase
from testing import *
from webob import Request, exc
from hgapp import utils

def request(meth='post', user=None, **kwargs):
    req = Request.blank('/')
    req.method = meth.upper()
    if user:
        kwargs['REMOTE_USER'] = user
    if 'baseui' in kwargs:
        req.hgapp_ui = kwargs.pop('baseui')
    req.environ.update(kwargs)
    return req

class TestPerms(TestCase):

    def get_request(self, section, **kwargs):
        ui = utils.get_ui(config['hg:%s' % section])
        req = request(baseui=ui, **kwargs)
        return req

    def test_public_anon(self):
        req = self.get_request('public', meth='get')
        eq_(utils.check_perm(req, 'pull'), None)
        req = self.get_request('public')
        self.assertRaises(exc.HTTPUnauthorized, utils.check_perm, req, 'push')

    def test_public_user(self):
        req = self.get_request('public', meth='get', user='toto')
        eq_(utils.check_perm(req, 'pull'), None)
        req = self.get_request('public', user='toto')
        eq_(utils.check_perm(req, 'push'), None)

    def test_private_anon(self):
        req = self.get_request('private', meth='get', user='toto')
        self.assertRaises(exc.HTTPUnauthorized, utils.check_perm, req, 'pull')
        req = self.get_request('private', user='toto')
        self.assertRaises(exc.HTTPUnauthorized, utils.check_perm, req, 'push')

    def test_private_user(self):
        req = self.get_request('private', meth='get', user='toto')
        self.assertRaises(exc.HTTPUnauthorized, utils.check_perm, req, 'pull')
        req = self.get_request('private', user='toto')
        self.assertRaises(exc.HTTPUnauthorized, utils.check_perm, req, 'push')

    def test_private_gawel(self):
        req = self.get_request('private', meth='get', user='gawel')
        eq_(utils.check_perm(req, 'pull'), None)
        req = self.get_request('private', user='gawel')
        eq_(utils.check_perm(req, 'push'), None)


class TestRepozePerms(TestPerms):

    def get_request(self, section, user=None, **kwargs):
        ui = utils.get_ui(config['hg:%s' % section])
        if user:
            if section == 'public':
                permissions = set(['allow_read', 'allow_push'])
            elif user == 'gawel':
                permissions = set(['allow_read', 'allow_push'])
            else:
                permissions = set()
            kwargs['repoze.what.credentials'] = dict(permissions=permissions)
        req = request(baseui=ui, user=user, **kwargs)
        return req

