# -*- coding: utf-8 -*-
from ConfigObject import ConfigObject
from nose.tools import *
import tempfile
import logging
import shutil
import time
import sys
import os

log = logging.getLogger('nose')

DIR_NAME = os.path.dirname(__file__)
package_repo = os.path.dirname(DIR_NAME)
CONFIG_FILE = os.path.join(DIR_NAME, 'test.ini')
os.environ['PASTE_CONFIG'] = CONFIG_FILE

from hgapp import config

TEMP_DIR = tempfile.mkdtemp(prefix='hgapp')

def setup_func():
    "set up test fixtures"
    if os.path.isdir(TEMP_DIR):
        shutil.rmtree(TEMP_DIR)
    os.makedirs(TEMP_DIR)

def teardown_func():
    "tear down test fixtures"
    if os.path.isdir(TEMP_DIR):
        shutil.rmtree(TEMP_DIR)

def timeit(func):
    name = func.func_name
    def wrapper(*args, **kwargs):
        st = time.time()
        func(*args, **kwargs)
        t = time.time() - st
        if args:
            self = args[0]
            log.warn('%s.%s() %2.3fs', self.__class__.__name__, t)
        else:
            log.warn('%s() %2.3fs', name, t)
    wrapper.func_name = name
    if hasattr(wrapper, '__name__'):
        wrapper.__name__ = func.func_name
    return wrapper

class TimeIt(object):
    """time middleware"""

    def __init__(self, app, msg=''):
        self.app = app
        self.msg = msg

    def __call__(self, environ, start_response):
        ct = time.time()
        result = self.app(environ, start_response)
        log.warn('%s %s %s %2.6fs', self.msg,
                      environ['REQUEST_METHOD'], environ['PATH_INFO'],
                      time.time() - ct)
        return result

