# -*- coding: utf-8 -*-
import os
import sys
import logging
import unittest
import subprocess
import hgapp
from testing import *
from hgapp.skin import make_skinned_app
from hgapp.pool import make_app
from webtest import TestApp, AppError

from mercurial.hgweb.request import wsgirequest
wsgirequest.drain = lambda *args, **kwargs: None

from hgapp import config

hgrc = os.path.join(package_repo, '.hg', 'hgrc')
hgrc_orig = open(hgrc).read()


class TestHgweb(unittest.TestCase):
    env = {}
    user = group = ''

    def setUp(self):
        if os.path.isdir(config['hg:config'].userdir):
            shutil.rmtree(config['hg:config'].userdir)
        self.app = TestApp(make_app({'__file__':CONFIG_FILE, 'here':DIR_NAME}))

    def tearDown(self):
        open(hgrc, 'w').write(hgrc_orig)

    def test_index(self):
        response = self.app.get('/public/summary')
        response.mustcontain('public')

    def test_push_unauthorize(self):
        self.assertRaises(AppError, self.app.post, '/public', dict(cmd='unbundle', heads='tip'))

    def test_unauthorize(self):
        self.assertRaises(AppError, self.app.get, '/private')

class TestHgwebAuth(TestHgweb):
    env = {'REMOTE_USER':'gawel'}

    def test_read(self):
        response = self.app.get('/public/summary', extra_environ=self.env)
        response.mustcontain('/public')

    def test_read_private(self):
        response = self.app.get('/private/summary', extra_environ=self.env)
        response.mustcontain('/private')

    def test_unauthorize(self):
        self.assertRaises(AppError, self.app.get, '/closed', extra_environ=self.env)


class TestHgwebdir(TestHgweb):

    def test_read(self):
        response = self.app.get('/public_dir')
        response.mustcontain('/MercurialApp')

    def test_read_private(self):
        self.assertRaises(AppError, self.app.post, '/private_dir/MercurialApp')

    def test_push_unauthorize(self):
        self.assertRaises(AppError, self.app.post, '/public_dir/MercurialApp', dict(cmd='unbundle', heads='tip'))

    def test_subrepo(self):
        response = self.app.get('/public_dir/MercurialApp')
        response.mustcontain('MercurialApp')

class TestHgwebdirAuth(TestHgwebdir):
    env = {'REMOTE_USER':'gawel'}

    def test_read(self):
        response = self.app.get('/public_dir', extra_environ=self.env)
        response.mustcontain('/MercurialApp')

    def test_read_private(self):
        response = self.app.get('/private_dir', extra_environ=self.env)
        response.mustcontain('/MercurialApp')

    def test_push_unauthorize(self):
        self.assertRaises(AppError, self.app.post, '/private_dir/MercurialApp', dict(cmd='unbundle', heads='tip'), extra_environ=self.env)

    def test_unauthorize(self):
        self.assertRaises(AppError, self.app.get, '/closed_dir', extra_environ=self.env)

    def test_unauthorize_subrepo(self):
        self.assertRaises(AppError, self.app.get, '/closed_dir/MercurialApp', extra_environ=self.env)


class TestSkin(TestHgwebdir):

    def setUp(self):
        if os.path.isdir(config['hg:config'].userdir):
            shutil.rmtree(config['hg:config'].userdir)
        self.app = TestApp(make_skinned_app({'__file__':CONFIG_FILE, 'here':DIR_NAME}))


    def test_read(self):
        response = self.app.get('/public_dir', extra_environ=self.env)
        response.mustcontain('/MercurialApp')
