# -*- coding: utf-8 -*-
from repoze.who.plugins.htpasswd import HTPasswdPlugin, crypt_check
from repoze.who.plugins.basicauth import BasicAuthPlugin
from repoze.who.plugins.form import FormPluginBase
from repoze.who.plugins.auth_tkt import AuthTktCookiePlugin
from repoze.what.plugins.ini import INIGroupAdapter
from repoze.what.plugins.ini import INIPermissionsAdapter
from repoze.what.middleware import setup_auth
import logging

log = logging.getLogger(__name__)

class BasicAuthWithCookie(FormPluginBase, BasicAuthPlugin):

    def __init__(self, realm, rememberer_name):
        self.realm = realm
        self.rememberer_name = rememberer_name

    def forget(self, environ, identity):
        FormPluginBase.forget(self, environ, identity)
        BasicAuthPlugin.forget(self, environ, identity)

class EmptyPlugin(object):
    def find_sections(self, *args):
        return ()

def make_ini(app, global_config, **local_conf):

    passwd = local_conf.get('passwd')
    htpasswd = HTPasswdPlugin(passwd, crypt_check)
    basicauth = BasicAuthWithCookie(local_conf.get('basicauth', 'Private repository'), '__ac')
    cookie = AuthTktCookiePlugin(local_conf.get('secret', 'mlklkmmmklkf354g543g4dq5g1dq43'))

    identifiers = [('__ac',cookie), ('basicauth', basicauth)]
    authenticators = [('htpasswd', htpasswd)]
    challengers = [('basicauth', basicauth)]
    mdproviders = []

    if 'groups' in local_conf:
        groups = {'all_groups': INIGroupAdapter(local_conf.get('groups'))}
    else:
        groups = {'all_groups': EmptyPlugin()}
    permissions = {'all_perms': EmptyPlugin()}

    middleware = setup_auth(
        app = app,
        group_adapters = groups,
        permission_adapters = permissions,
        identifiers = identifiers,
        authenticators = authenticators,
        challengers = challengers,
        mdproviders = mdproviders,
        log_level = logging.WARN
    )
    return middleware

